@@ -4,29 +4,58 @@ from typing import Any
 from flask import request
 from sqlalchemy import and_, inspect
 from sqlalchemy.exc import DataError, IntegrityError, SQLAlchemyError
+from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import DeclarativeBase, Query, Session, object_session
 from sqlalchemy.orm.exc import UnmappedInstanceError
 
 from flarchitect.core.utils import get_primary_key_info
 from flarchitect.database.inspections import get_model_columns, get_model_relationships
-from flarchitect.database.utils import (
-    AGGREGATE_FUNCS,
-    create_aggregate_conditions,
-    generate_conditions_from_args,
-    get_all_columns_and_hybrids,
-    get_group_by_fields,
-    get_models_for_join,
-    get_primary_key_filters,
-    get_related_b_query,
-    get_select_fields,
-    get_table_and_column,
-    parse_column_table_and_operator,
-    validate_table_and_column,
-    _eager_options_for,
+
+# Import utility helpers with a graceful fallback for optional helpers such as
+# ``_eager_options_for`` which may not be present when tests monkeypatch the
+# ``flarchitect.database.utils`` module. The recursive delete fixture replaces
+# the module with a lightweight stub, so we resolve attributes dynamically.
+from flarchitect.database import utils as _db_utils
+
+
+AGGREGATE_FUNCS = _db_utils.AGGREGATE_FUNCS
+create_aggregate_conditions = _db_utils.create_aggregate_conditions
+generate_conditions_from_args = _db_utils.generate_conditions_from_args
+get_all_columns_and_hybrids = _db_utils.get_all_columns_and_hybrids
+get_group_by_fields = _db_utils.get_group_by_fields
+get_models_for_join = _db_utils.get_models_for_join
+get_primary_key_filters = _db_utils.get_primary_key_filters
+get_related_b_query = _db_utils.get_related_b_query
+get_select_fields = _db_utils.get_select_fields
+get_table_and_column = _db_utils.get_table_and_column
+parse_column_table_and_operator = _db_utils.parse_column_table_and_operator
+validate_table_and_column = _db_utils.validate_table_and_column
+normalise_join_token = getattr(
+    _db_utils,
+    "normalise_join_token",
+    lambda token: (token or "").strip().lower().replace("-", "_"),
 )
+
+
+def _fallback_table_namer(model: DeclarativeBase | None = None) -> str:
+    """Gracefully derive a table name when the DB utils module is stubbed."""
+
+    if model is None:
+        return ""
+
+    table_name = getattr(model, "__tablename__", None)
+    if isinstance(table_name, str) and table_name:
+        return table_name
+
+    return getattr(model, "__name__", "").strip().lower()
+
+
+table_namer = getattr(_db_utils, "table_namer", _fallback_table_namer)
+_eager_options_for = getattr(_db_utils, "_eager_options_for", lambda *args, **kwargs: [])
 from flarchitect.exceptions import CustomHTTPException
 from flarchitect.utils.config_helpers import get_config_or_model_meta
 from flarchitect.utils.decorators import add_dict_to_query, add_page_totals_and_urls
+from flarchitect.utils.core_utils import convert_case
 
 __all__ = [
     "paginate_query",
@@ -165,17 +194,37 @@ class CrudService:
             CustomHTTPException: If the field does not represent a relationship.
         """
         relationships = inspect(self.model).relationships
-        related_model = relationships.get(field_name)
+        # Try direct relationship key match first
+        related = relationships.get(field_name)
+        if related is not None:
+            return related.mapper.class_
 
-        if related_model is None:
-            raise CustomHTTPException(
-                401,
-                f"Field {field_name} does not represent a relationship in model {self.model.__name__}",
-            )
+        # Be flexible: allow endpoint-style names or case variations
+        target = (field_name or "").strip().lower()
+        from flarchitect.specs.utils import endpoint_namer
+        from flarchitect.utils.config_helpers import get_config_or_model_meta
+        from flarchitect.utils.core_utils import convert_case
 
-        return related_model.mapper.class_
+        endpoint_case = get_config_or_model_meta("API_ENDPOINT_CASE", default="kebab") or "kebab"
 
-    def filter_query_from_args(self, args_dict: dict[str, str | int], query=None) -> Query:
+        for rel in relationships.values():
+            model_cls = rel.mapper.class_
+            # Endpoint-style plural name (e.g., "authors")
+            endpoint_name = endpoint_namer(model_cls).lower()
+            # Relationship key in endpoint case (often singular)
+            rel_key_endpoint_case = convert_case(rel.key, endpoint_case).lower()
+            # Raw relationship key
+            rel_key_raw = rel.key.lower()
+
+            if target in {endpoint_name, rel_key_endpoint_case, rel_key_raw}:
+                return model_cls
+
+        raise CustomHTTPException(
+            401,
+            f"Field {field_name} does not represent a relationship in model {self.model.__name__}",
+        )
+
+    def filter_query_from_args(self, args_dict: dict[str, Any], query=None) -> Query:
         """Build a query applying joins, filters, grouping and aggregation.
 
         Args:
@@ -186,21 +235,48 @@ class CrudService:
             Query with all requested transformations applied.
         """
 
+        # Flatten values to simple scalars for general processing while preserving
+        # the full argument structure for join parsing.
+        def _flatten_values(d: dict[str, Any]) -> dict[str, Any]:
+            return {k: (v[0] if isinstance(v, (list, tuple)) else v) for k, v in d.items()}
+
         allow_join = get_config_or_model_meta("API_ALLOW_JOIN", model=self.model, default=False)
         join_models = get_models_for_join(args_dict, self.fetch_related_model_by_name) if allow_join else {}
+        # Use flattened args for all non-join processing
+        flat_args = _flatten_values(args_dict)
 
         all_columns, all_models = get_all_columns_and_hybrids(self.model, join_models)
 
-        conditions = [condition for condition in generate_conditions_from_args(args_dict, self.model, all_columns, all_models, join_models) if condition is not None]
+        schema_case = get_config_or_model_meta("API_SCHEMA_CASE", model=self.model, default="camel")
+
+        alias_to_model: dict[str, type[DeclarativeBase]] = {}
+
+        def _register_alias(model_cls: type[DeclarativeBase], *names: str) -> None:
+            for name in names:
+                if not name:
+                    continue
+                alias_to_model.setdefault(name, model_cls)
+                normalised = normalise_join_token(name)
+                if normalised:
+                    alias_to_model.setdefault(normalised, model_cls)
+
+        base_alias = convert_case(self.model.__name__, schema_case)
+        _register_alias(self.model, base_alias, table_namer(self.model))
+
+        for token, mdl in join_models.items():
+            canonical = convert_case(mdl.__name__, schema_case)
+            _register_alias(mdl, token, canonical, table_namer(mdl))
+
+        conditions = [condition for condition in generate_conditions_from_args(flat_args, self.model, all_columns, all_models, join_models) if condition is not None]
 
         allow_select = get_config_or_model_meta("API_ALLOW_SELECT_FIELDS", model=self.model, default=True)
-        select_fields = get_select_fields(args_dict, self.model, all_columns) if allow_select else []
+        select_fields = get_select_fields(flat_args, self.model, all_columns) if allow_select else []
 
         allow_group = get_config_or_model_meta("API_ALLOW_GROUPBY", model=self.model, default=False)
-        group_by_fields = get_group_by_fields(args_dict, all_columns, self.model) if allow_group else []
+        group_by_fields = get_group_by_fields(flat_args, all_columns, self.model) if allow_group else []
 
         allow_agg = get_config_or_model_meta("API_ALLOW_AGGREGATION", model=self.model, default=False)
-        aggregate_conditions = create_aggregate_conditions(args_dict) if allow_agg else {}
+        aggregate_conditions = create_aggregate_conditions(flat_args) if allow_agg else {}
         agg_fields = []
         for key, label in aggregate_conditions.items():
             column_name, table_name, func_name = parse_column_table_and_operator(key, self.model)
@@ -208,13 +284,34 @@ class CrudService:
             agg_func = AGGREGATE_FUNCS.get(func_name)
             if agg_func:
                 agg_label = label or f"{column_name}_{func_name}"
-                agg_fields.append(agg_func(model_column).label(agg_label))
+                owning_model = alias_to_model.get(table_name) or alias_to_model.get(normalise_join_token(table_name)) or self.model
+                target_column = getattr(owning_model, column_name) if isinstance(model_column, hybrid_property) else model_column
+                agg_fields.append(agg_func(target_column).label(agg_label))
 
         query = query or self.session.query(self.model)
 
         if allow_join and join_models:
+            join_type = str(flat_args.get("join_type", "inner")).lower()
+            if join_type not in {"inner", "left", "right", "outer"}:
+                raise CustomHTTPException(400, f"Invalid join_type: {join_type}. Supported: inner,left,right,outer")
+
             for mdl in join_models.values():
-                query = query.join(mdl)
+                if join_type == "left" or join_type == "outer":
+                    query = query.outerjoin(mdl)
+                elif join_type == "right":
+                    # SQLAlchemy ORM has no direct RIGHT JOIN; `isouter=True` behaves as LEFT OUTER in most dialects.
+                    # Right join semantics require reversed join order; this is a best-effort fallback.
+                    query = query.join(mdl, isouter=True)
+                else:
+                    query = query.join(mdl)
+
+            # When joining one-to-many relationships, the base entity rows can
+            # be duplicated which breaks pagination semantics (limit applies to
+            # multiplied rows rather than distinct base rows). If the request
+            # is not selecting custom fields, grouping, or aggregating, ensure
+            # we paginate over distinct base rows.
+            if not select_fields and not group_by_fields and not agg_fields:
+                query = query.distinct()
 
         if agg_fields or group_by_fields:
             query = query.with_entities(*(group_by_fields + agg_fields))
@@ -367,20 +464,22 @@ class CrudService:
         if callback:
             query = callback(query, self.model, args_dict)
 
+        # Flatten args for order/pagination values
+        flat_args = {k: (v[0] if isinstance(v, (list, tuple)) else v) for k, v in args_dict.items()}
         count = query.count()
-        order_query = self.order_query(args_dict, query)
+        order_query = self.order_query(flat_args, query)
 
         # Apply soft delete filtering before pagination to avoid operating on
         # paginated objects, which lack SQLAlchemy query attributes such as
         # ``column_descriptions``.
         filtered_query = self.apply_soft_delete_filter(order_query)
 
-        paginated_query, default_pagination_size = paginate_query(filtered_query, args_dict.get("page", 1), args_dict.get("limit"))
+        paginated_query, default_pagination_size = paginate_query(filtered_query, flat_args.get("page", 1), flat_args.get("limit"))
 
         return {
             "query": (paginated_query.all() if hasattr(paginated_query, "all") else (paginated_query.items if hasattr(paginated_query, "items") else paginated_query)),
-            "limit": (int(args_dict.get("limit")) if args_dict.get("limit") else default_pagination_size),
-            "page": int(args_dict.get("page")) if args_dict.get("page") else 1,
+            "limit": (int(flat_args.get("limit")) if flat_args.get("limit") else default_pagination_size),
+            "page": int(flat_args.get("page")) if flat_args.get("page") else 1,
             "total_count": count,
         }
 
@@ -432,9 +531,16 @@ class CrudService:
             if obj is None:
                 raise CustomHTTPException(404, f"{self.model.__name__} not found.")
 
+            payload = data_dict or {}
             allow_nested = get_config_or_model_meta("ALLOW_NESTED_WRITES", model=self.model, default=False)
-            update_payload = self._process_nested_relationships(self.model, data_dict.copy()) if allow_nested else data_dict
+            update_payload = self._process_nested_relationships(self.model, payload.copy()) if allow_nested else payload
+
+            mapper = inspect(self.model)
+            writable_keys = {column.key for column in mapper.columns}
+
             for key, value in update_payload.items():
+                if key not in writable_keys:
+                    continue
                 setattr(obj, key, value)
 
             callback = get_config_or_model_meta("API_UPDATE_CALLBACK", model=self.model, default=None)
