@@ -1,6 +1,7 @@
 import datetime
 import uuid
-from typing import Any
+from decimal import Decimal
+from typing import Any, get_type_hints, get_origin, get_args
 
 from flask import request
 from marshmallow import Schema, ValidationError, fields, post_dump, pre_dump
@@ -32,6 +33,7 @@ from sqlalchemy import (
     Time,
 )
 from sqlalchemy.dialects.postgresql import JSON, JSONB
+from sqlalchemy.types import JSON as SQLAlchemyJSON
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import ColumnProperty, RelationshipProperty, class_mapper
 from sqlalchemy.orm import exc as orm_exc
@@ -80,6 +82,23 @@ class EnumField(fields.Field):
             raise ValidationError(f"Invalid enum value. Expected one of: {valid}.") from e
 
 
+class NumericNumber(fields.Decimal):
+    """Serialise ``Numeric`` values as JSON-native numbers."""
+
+    def __init__(self, *args, **kwargs) -> None:
+        kwargs.setdefault("as_string", False)
+        kwargs.setdefault("allow_nan", False)
+        super().__init__(*args, **kwargs)
+
+    def _serialize(self, value, attr, obj, **kwargs):  # type: ignore[override]
+        serialised = super()._serialize(value, attr, obj, **kwargs)
+        if serialised is None:
+            return None
+        if isinstance(serialised, Decimal):
+            return float(serialised)
+        return serialised
+
+
 # Mapping between SQLAlchemy types and Marshmallow fields
 type_mapping = {
     Integer: fields.Int,
@@ -94,7 +113,8 @@ type_mapping = {
     TIMESTAMP: fields.DateTime,  # Added TIMESTAMP
     JSON: fields.Raw,
     JSONB: fields.Raw,
-    Numeric: fields.Decimal,
+    SQLAlchemyJSON: fields.Raw,
+    Numeric: NumericNumber,
     BigInteger: fields.Int,
     LargeBinary: fields.Str,  # Consider using fields.Raw for binary data
     Enum: EnumField,  # Consider fields.Enum for stricter validation
@@ -270,8 +290,10 @@ class AutoSchema(Base):
         mapper_property: RelationshipProperty,
     ):
         """Handle adding a relationship field to the schema."""
-        if not get_config_or_model_meta("API_ADD_RELATIONS", model=self.model, default=True):
-            return
+        add_relations = get_config_or_model_meta("API_ADD_RELATIONS", model=self.model, default=True)
+        allow_join = get_config_or_model_meta("API_ALLOW_JOIN", model=self.model, default=False)
+
+        # Respect per-request opt-out for dumping relationships entirely
         try:
             if request.args.get("dump_relationships") in ["false", "False", "0"]:
                 return
@@ -280,6 +302,51 @@ class AutoSchema(Base):
             # initialised
             pass
 
+        # If relations are globally disabled, still allow explicit inclusion
+        # when joins are both allowed and requested for this relationship.
+        if not add_relations:
+            if not allow_join:
+                return
+            def _collect_join_tokens() -> set[str]:
+                try:
+                    values = []
+                    # all values for join and legacy join_models
+                    values.extend(request.args.getlist("join"))
+                    values.extend(request.args.getlist("join_models"))
+                    tokens: set[str] = set()
+                    for v in values:
+                        for raw in str(v).split(","):
+                            tok = raw.strip().lower().replace("-", "_")
+                            if tok:
+                                tokens.add(tok)
+                    # also treat additional keys as join tokens (when names match)
+                    reserved = {"join", "join_models", "join_type", "fields", "groupby", "orderby", "order_by", "page", "limit", "dump", "format", "include_deleted", "cascade_delete"}
+                    for k in request.args.keys():
+                        if k not in reserved and "__" not in k and "." not in k:
+                            tokens.add(k.strip().lower().replace("-", "_"))
+                    return tokens
+                except Exception:
+                    return set()
+
+            tokens = _collect_join_tokens()
+
+            # Compute candidate tokens for this relationship
+            from flarchitect.specs.utils import endpoint_namer
+            from flarchitect.utils.core_utils import convert_case
+
+            endpoint_case = get_config_or_model_meta("API_ENDPOINT_CASE", default="kebab") or "kebab"
+            related_model = mapper_property.property.mapper.class_
+            endpoint_name = get_config_or_model_meta("API_ENDPOINT_NAMER", related_model, default=endpoint_namer)(related_model).lower()
+            rel_key_endpoint_case = convert_case(original_attribute, endpoint_case).lower()
+            rel_key_raw = original_attribute.lower()
+            candidates = {endpoint_name, rel_key_endpoint_case, rel_key_raw}
+
+            requested = not tokens.isdisjoint(candidates)
+
+            if not requested:
+                return
+
+        # Either relations are enabled or an explicit join requested this one
         self.add_relationship_field(attribute, original_attribute, mapper_property)
 
     def _handle_column(self, attribute: str, original_attribute: str, mapper_property: ColumnProperty):
@@ -300,17 +367,44 @@ class AutoSchema(Base):
         if self._should_skip_attribute(attribute):
             return
 
-        field_type = type_mapping.get(field_type, fields.Str) if field_type else fields.Str
+        model_cls = None
+        if isinstance(self.model, type):
+            model_cls = self.model
+        elif self.model is not None:
+            model_cls = type(self.model)
 
-        # Check if the attribute has a setter method
-        has_setter = (
-            hasattr(type(self.model), original_attribute) and isinstance(getattr(type(self.model), original_attribute), property) and getattr(type(self.model), original_attribute).fset is not None
-        )
+        descriptor = getattr(model_cls, original_attribute, None) if model_cls is not None else None
+
+        if field_type is None and descriptor is not None:
+            # SQLAlchemy hybrid properties expose annotations on ``fget``
+            fget = getattr(descriptor, "fget", None)
+            if fget is not None:
+                try:
+                    field_type = get_type_hints(fget).get("return")
+                except Exception:
+                    field_type = None
+                if field_type is None:
+                    field_type = getattr(fget, "__annotations__", {}).get("return")
+
+        if isinstance(field_type, type) and issubclass(field_type, fields.Field):
+            schema_field_cls = field_type
+        else:
+            if field_type is not None:
+                origin = get_origin(field_type)
+                if origin is not None:
+                    args = [arg for arg in get_args(field_type) if arg is not type(None)]
+                    field_type = args[0] if len(args) == 1 else origin
+
+            resolved_field = type_mapping.get(field_type, fields.Str) if field_type else fields.Str
+            schema_field_cls = resolved_field if isinstance(resolved_field, type) else fields.Str
+
+        # Check if the attribute has a setter method (properties/hybrids expose ``fset``)
+        has_setter = descriptor is not None and getattr(descriptor, "fset", None) is not None
 
         # If there's no setter, mark it as dump_only
         field_args = {"dump_only": not has_setter}
 
-        self.add_to_fields(original_attribute, field_type(data_key=attribute, **field_args), load=False)
+        self.add_to_fields(original_attribute, schema_field_cls(data_key=attribute, **field_args), load=False)
 
         self._update_field_metadata(original_attribute)
 
@@ -356,6 +450,11 @@ class AutoSchema(Base):
             # Get additional attrs for the field based on the column's properties
             field_args = self._get_column_field_attrs(original_attribute, column_type)
 
+            if issubclass(field_type, NumericNumber) and isinstance(column_type, Numeric):
+                scale = getattr(column_type, "scale", None)
+                if scale is not None:
+                    field_args.setdefault("places", scale)
+
             # Instantiate the field
             field = field_type(data_key=attribute, **field_args)
 
@@ -584,170 +683,143 @@ class AutoSchema(Base):
         original_attribute: str,
         relationship_property: RelationshipProperty,
     ) -> None:
-        """Add a schema field for a SQLAlchemy relationship.
+        """Add serialisation fields for a SQLAlchemy relationship."""
 
-        Args:
-            attribute (str): Name of the field exposed by the schema.
-            original_attribute (str): Name of the relationship attribute on the model.
-            relationship_property (RelationshipProperty): SQLAlchemy relationship
-                descriptor.
-
-        Returns:
-            None
-
-        Side Effects:
-            Modifies schema field mappings and updates field metadata. May add
-            additional load-only fields when nested writes are enabled.
-        """
         allow_nested_writes = get_config_or_model_meta("ALLOW_NESTED_WRITES", model=self.model, default=False)
         max_depth = 2 if allow_nested_writes else 1
         current_depth = self.context.get("current_depth", 0)
 
         if current_depth >= max_depth:
-            # # Exceeded maximum depth, include only the primary key reference
-            # related_model = relationship_property.mapper.class_
-            # primary_key = inspect(related_model).primary_key[0]
-            # field_type = type_mapping.get(type(primary_key.type), fields.Integer)
-            #
-            # self.add_to_fields(
-            #     original_attribute, field_type(data_key=attribute)
-            # )
             return
 
+        input_schema, output_schema = get_input_output_from_model_or_make(
+            relationship_property.mapper.class_,
+            context={"current_depth": current_depth + 1},
+        )
+
+        relationship_prop = relationship_property.property
+        field_args = {"dump_only": not relationship_prop.viewonly}
+
+        try:
+            dump_override = request.args.get("dump")
+        except RuntimeError:
+            dump_override = None
+        dump_override = (dump_override or "").strip().lower()
+        raw_dump = get_config_or_model_meta("API_SERIALIZATION_TYPE", self.model, default="url")
+        if raw_dump is False:
+            configured_dump = "json"
         else:
-            # Include the nested schema and increment the depth
-            input_schema, output_schema = get_input_output_from_model_or_make(
-                relationship_property.mapper.class_,
-                context={"current_depth": current_depth + 1},
-            )
+            configured_dump = str(raw_dump or "url").strip().lower()
+        dump_type = dump_override if dump_override in {"url", "json", "dynamic", "hybrid"} else configured_dump
 
-            # Access the RelationshipProperty to get 'viewonly' and 'uselist'
-            relationship_prop = relationship_property.property
-            field_args = {"dump_only": not relationship_prop.viewonly}
+        raw_depth = get_config_or_model_meta("API_SERIALIZATION_DEPTH", model=self.model, default=None)
 
-            # Determine the serialization type
-            dump_type = get_config_or_model_meta("API_SERIALIZATION_TYPE", self.model, default="url")
+        serialization_depth: int | None
+        if raw_depth in (None, "") or raw_depth is False:
+            serialization_depth = None
+        else:
+            try:
+                serialization_depth = int(raw_depth)
+            except (TypeError, ValueError):
+                serialization_depth = None
 
-            if dump_type == "url":
-                if relationship_prop.uselist:
-                    # Serialize as a list of URLs
-                    self.add_to_fields(
-                        attribute,
-                        fields.Function(
-                            lambda obj: self.get_many_url(obj, original_attribute, input_schema),
-                            **field_args,
-                        ),
-                        load=False,
-                    )
-                    field_name = attribute
-                else:
-                    # Serialize as a single URL
-                    self.add_to_fields(
-                        attribute,
-                        fields.Function(
-                            lambda obj: self.get_url(obj, original_attribute, input_schema),
-                            **field_args,
-                        ),
-                        load=False,
-                    )
-                    field_name = attribute
-            elif dump_type == "json":
-                if relationship_prop.uselist:
-                    # Always dump as a list of nested schemas
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.List(fields.Nested(output_schema), **field_args),
-                        load=False,
-                    )
-                    field_name = original_attribute
-                else:
-                    # Always dump as a nested schema
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.Nested(output_schema, **field_args),
-                        load=False,
-                    )
-                    field_name = original_attribute
-            elif dump_type == "dynamic":
-                try:
-                    _ = request.args
-                except RuntimeError:
-                    return
+        if serialization_depth is not None and serialization_depth < 0:
+            serialization_depth = None
 
-                if request and request.args.get("join"):
-                    joins = request.args.get("join")
-
-                    if not joins:
-                        return  # Exit early if no join parameter is provided
-
-                    url_naming_function = get_config_or_model_meta(
-                        "API_ENDPOINT_NAMER",
-                        relationship_property.mapper.class_,
-                        default=endpoint_namer,
-                    )
-                    name = url_naming_function(output_schema)
-
-                    if not iterable(joins):
-                        joins = [joins]
-
-                    if name in joins:
-                        if relationship_prop.uselist:
-                            # Always dump as a list of nested schemas
-                            self.add_to_fields(
-                                original_attribute,
-                                fields.List(fields.Nested(output_schema), **field_args),
-                                load=False,
-                            )
-                            field_name = original_attribute
-                        else:
-                            # Always dump as a nested schema
-                            self.add_to_fields(
-                                original_attribute,
-                                fields.Nested(output_schema, **field_args),
-                                load=False,
-                            )
-                            field_name = original_attribute
-                    else:
-                        return
-                else:
-                    return
-
-            elif dump_type == "hybrid":
-                if relationship_prop.uselist:
-                    # Serialize as a list of URLs
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.Function(
-                            lambda obj: self.get_many_url(obj, attribute, input_schema),
-                            **field_args,
-                        ),
-                        load=False,
-                    )
-                    field_name = original_attribute
-                else:
-                    # Serialize as a nested schema
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.Nested(output_schema, **field_args),
-                        load=False,
-                    )
-                    field_name = original_attribute
+        def add_url_field() -> str:
+            if relationship_prop.uselist:
+                self.add_to_fields(
+                    attribute,
+                    fields.Function(lambda obj: self.get_many_url(obj, original_attribute, input_schema), **field_args),
+                    load=False,
+                )
             else:
-                # Fallback to JSON serialization if an unknown dump_type is provided
-                if relationship_prop.uselist:
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.List(fields.Nested(output_schema), **field_args),
-                        load=False,
-                    )
-                    field_name = original_attribute
-                else:
-                    self.add_to_fields(
-                        original_attribute,
-                        fields.Nested(output_schema, **field_args),
-                        load=False,
-                    )
-                    field_name = original_attribute
+                self.add_to_fields(
+                    attribute,
+                    fields.Function(lambda obj: self.get_url(obj, original_attribute, input_schema), **field_args),
+                    load=False,
+                )
+            return attribute
+
+        def add_nested_field() -> str:
+            if relationship_prop.uselist:
+                self.add_to_fields(
+                    original_attribute,
+                    fields.List(fields.Nested(output_schema), **field_args),
+                    load=False,
+                )
+            else:
+                self.add_to_fields(
+                    original_attribute,
+                    fields.Nested(output_schema, **field_args),
+                    load=False,
+                )
+            return original_attribute
+
+        def collect_join_tokens() -> set[str]:
+            try:
+                values: list[str] = []
+                values.extend(request.args.getlist("join"))
+                values.extend(request.args.getlist("join_models"))
+                tokens: set[str] = set()
+                for v in values:
+                    for raw in str(v).split(","):
+                        tok = raw.strip().lower().replace("-", "_")
+                        if tok:
+                            tokens.add(tok)
+                reserved = {
+                    "join",
+                    "join_models",
+                    "join_type",
+                    "fields",
+                    "groupby",
+                    "orderby",
+                    "order_by",
+                    "page",
+                    "limit",
+                    "dump",
+                    "format",
+                    "include_deleted",
+                    "cascade_delete",
+                }
+                for key in request.args.keys():
+                    if key not in reserved and "__" not in key and "." not in key:
+                        tokens.add(key.strip().lower().replace("-", "_"))
+                return tokens
+            except Exception:
+                return set()
+
+        def depth_exceeded() -> bool:
+            return serialization_depth is not None and current_depth >= serialization_depth
+
+        if dump_type == "url":
+            field_name = add_url_field()
+        elif dump_type == "json":
+            field_name = add_url_field() if depth_exceeded() else add_nested_field()
+        elif dump_type == "dynamic":
+            tokens = collect_join_tokens()
+            if not tokens:
+                return
+            related_model = relationship_property.mapper.class_
+            endpoint_case = get_config_or_model_meta("API_ENDPOINT_CASE", default="kebab") or "kebab"
+            endpoint_name = get_config_or_model_meta("API_ENDPOINT_NAMER", related_model, default=endpoint_namer)(related_model).lower()
+            rel_key_endpoint_case = convert_case(original_attribute, endpoint_case).lower()
+            rel_key_raw = original_attribute.lower()
+            requested = not tokens.isdisjoint({endpoint_name, rel_key_endpoint_case, rel_key_raw})
+            if not requested:
+                return
+
+            # An explicit join request should inline the related payload even when the
+            # configured depth would normally fall back to URLs. This keeps
+            # ``API_SERIALIZATION_TYPE="dynamic"`` consistent with ``?dump=dynamic``.
+            field_name = add_nested_field()
+        elif dump_type == "hybrid":
+            if relationship_prop.uselist:
+                field_name = add_url_field()
+            else:
+                field_name = add_url_field() if depth_exceeded() else add_nested_field()
+        else:
+            field_name = add_nested_field()
 
         self._update_field_metadata(field_name)
 
